#ifndef PYTHONIC_NUMPY_FMIN_REDUCE_HPP
#define PYTHONIC_NUMPY_FMIN_REDUCE_HPP

#define UFUNC_NAME fmin
#define UFUNC_INAME imin
#include "pythonic/include/numpy/fmin/reduce.hpp"
#include "pythonic/numpy/ufunc_reduce.hpp"
#undef UFUNC_NAME
#undef UFUNC_INAME

#endif
