#ifndef PYTHONIC_INCLUDE_OPERATOR_ISUB_HPP
#define PYTHONIC_INCLUDE_OPERATOR_ISUB_HPP

#define OPERATOR_NAME isub
#define OPERATOR_SYMBOL -
#define OPERATOR_ISYMBOL -=
#include "pythonic/include/operator_/icommon.hpp"

#endif
