print("import: 'omp'")
import omp

print("import: 'pythran'")
import pythran

print("import: 'pythran.analyses'")
import pythran.analyses

print("import: 'pythran.optimizations'")
import pythran.optimizations

print("import: 'pythran.transformations'")
import pythran.transformations

print("import: 'pythran.types'")
import pythran.types

