#ifndef PYTHONIC_BUILTIN_PYTHRAN_IS_NONE_HPP
#define PYTHONIC_BUILTIN_PYTHRAN_IS_NONE_HPP

#include "pythonic/include/builtins/pythran/is_none.hpp"
#include "pythonic/utils/functor.hpp"
#include "pythonic/types/NoneType.hpp"

PYTHONIC_NS_BEGIN

namespace builtins
{

  namespace pythran
  {
  }
}

PYTHONIC_NS_END

#endif
