#ifndef PYTHONIC_IO__IO_TEXTIOWRAPPER_WRITELINES_HPP
#define PYTHONIC_IO__IO_TEXTIOWRAPPER_WRITELINES_HPP

#include "pythonic/include/io/_io/TextIOWrapper/writelines.hpp"
#include "pythonic/builtins/file/writelines.hpp"

#endif
