#ifndef PYTHONIC_INCLUDE_NUMPY_RANDOM_RANF_HPP
#define PYTHONIC_INCLUDE_NUMPY_RANDOM_RANF_HPP

#include "pythonic/include/numpy/random/random.hpp"
#include "pythonic/include/utils/functor.hpp"

PYTHONIC_NS_BEGIN
namespace numpy
{
  namespace random
  {
    USING_FUNCTOR(ranf, random);
  }
}
PYTHONIC_NS_END

#endif
