#ifndef PYTHONIC_OPERATOR_IRSHIFT_HPP
#define PYTHONIC_OPERATOR_IRSHIFT_HPP

#include "pythonic/include/operator_/irshift.hpp"

#define OPERATOR_NAME irshift
#define OPERATOR_SYMBOL >>
#define OPERATOR_ISYMBOL >>=

#include "pythonic/operator_/icommon.hpp"

#endif
